#include "QTools.h"

#include <algorithm>
#include <fcntl.h>
#include <fstream>
#include <io.h>
#include <iostream>
#include <math.h>
#include <process.h>
#include <stdio.h>
#include <time.h>

using namespace std;


// 2^64 = 18446744073709551616
// 2^63 = 9223372036854775808
// 2^62 = 4611686018427387904
// 2^61 = 2305843009213693952
// 2^32 = 4294967296
// lg(2)/lg(5) = 0.43067655807339305067010656876428
// lg(2)/lg(7) = 0,356207187108022176514177078001291
// 5^27 = 7450580596923828125
// 5^26 = 1490116119384765625
// 5^13 = 1220703125
// 7^11 = 1977326743
/**************************************************
  Test Results:
    Number of shots = 1000000;
    Number of intervals = 1000;
    Max shots = 1019 (1.9%);
    Min shots = 981 (1.9%);
*/

static unsigned __int64 RandM64 = 1220703125;
static const unsigned __int64 a064 = 4294967296;
//static const unsigned __int64 a064 = 4294967295;
static unsigned __int64 an64;
static bool isRNGInitialized = false;

static double Rand_M = 30517578125.0;     // 5^15
static const double a0 = 1.4551915228366851806640625e-11;   // 2^(-36)
static double an, an_floor;

const double aspect_1 = 1.0/fabs((double)0x7fffffff);

void QInitRand (unsigned seed) {
	if (isRNGInitialized == false) {
		cout<<"Initialization of random numbers generator\n";
		unsigned useless = seed % 32768;

		an64 = a064 * RandM64;

	/*  cout<<"--== Pseudorandom numbers generator information:\n"
				<<"RandM = "<<(unsigned int)RandM64<<endl
				<<"Seed = "<<(unsigned int)seed<<endl
				<<"Useless = "<<useless<<endl;/**/

		for (unsigned i=0; i<useless; i++) {
			QRand ();
		}
		isRNGInitialized = true;
	//  an64 += RandM64;
	}	// end of if (isRNGInitialized)
}

unsigned int QRand () {
  an64 *= RandM64;

//  return (unsigned)(an64>>15);  // 10.1% dispersion
//  return (unsigned)(an64>>16);  // 4.1% dispersion
//  return (unsigned)(an64>>17);  // 3% dispersion
//  return (unsigned)(an64>>18);  // 1.9% dispersion
//  return (unsigned)(an64>>19);  // 2% dispersion (2.2)
//  return (unsigned)(an64>>20);  // 2.1% dispersion
//  return (unsigned)(an64>>21);  // 2.6% dispersion (2.7)
  // works best because operation ">>22" shifts number so that least 22 digits
  // are erased and convertion to unsigned takes 32 least digits that are left
  // after shift
  return (unsigned)(an64>>22);  // 2.8% dispersion (2.1%)
//  return (unsigned)(an64>>23);  // 3% dispersion (10%)
//  return (unsigned)(an64>>24);  // 9.7% dispersion (10.2)
//  return (unsigned)(an64>>25);  // 10.5% dispersion
//  return (unsigned)(an64>>26);  // 9.5% dispersion
//  return (unsigned)(an64>>27);  // 9.4% dispersion
//  return (unsigned)(an64>>28);  // 12.7% dispersion
//  return (unsigned)(an64>>29);  // 11.5% dispersion
//  return (unsigned)(an64>>30);  // 10.8% dispersion
//  return (unsigned)(an64>>31);  // 10.6% dispersion
//  return (unsigned)(an64>>32);  // 10.7% dispersion

//  double temp = fabs((double)(int)(an64>>22))/fabs((double)0x7fffffff);
//  return QMAX_UNSIGNED * temp;
}

double QRandN () {
  an64 *= RandM64;
//  unsigned temp = unsigned(an64>>32);
  return fabs((double)(int)(an64>>22))*aspect_1;
}

bool QRandBool () {
  an64 *= RandM64;

  return ((unsigned)(an64>>22)>(unsigned)0x7fffffff)? true : false;
}

double QRandGauss () {
	double sum=0.0;
	for (int i=0; i<12; i++) {
		sum += QRandN ();
	}
	sum -= 6.0;	// center the [sum]
	return sum/6.0;	// fit output into the range [-1; +1]
}

////////////////////////////////////////////////////////////////////////////
// Miscelaneous functions definitions

int roulette (const __VECTOR_INT& argV, int INITIAL_VALUE) {
	//__VECTOR_INT tempV;
	//tempV.assign (argV.begin(), argV.end());

	int size = (int)argV.size();
	int temp, tempSum, i;

	tempSum = 0;
	for (i=0; i<size; i++) {
//		tempV[i] += INITIAL_VALUE;
		tempSum += argV[i];
	}
	tempSum += INITIAL_VALUE * size;

	// define element for selection using roulette
	temp = (int)(tempSum * QRandN() + 0.5);
	for (i=0; i<size; i++) {
		temp -= argV[i] + INITIAL_VALUE;
		if (temp <= 0) {
			// the i-th node is selected
			break;
		}
	}

	return i;
}

int roulette (const __VECTOR_DOUBLE& argV, int INITIAL_VALUE) {
	int size = (int)argV.size();
	int i;
	double temp, tempSum;

	tempSum = 0;
	for (i=0; i<size; i++) {
//		tempV[i] += INITIAL_VALUE;
		tempSum += argV[i];
	}
	tempSum += INITIAL_VALUE * size;

	// define element for selection using roulette
	temp = tempSum * QRandN();
	for (i=0; i<size; i++) {
		temp -= argV[i] + INITIAL_VALUE;
		if (temp <= 0) {
			// the i-th node is selected
			break;
		}
	}

	return i;
}

// checks whether the given matrix of bytes contains all-zeroes
bool isNull (const __TABLE_BYTE& argM) {
	if (!argM.empty()) {
		__TABLE_BYTE::const_iterator iterH = argM.begin(), hEnd = argM.end();
		__VECTOR_BYTE::const_iterator iterW, wEnd;

		for (; iterH != hEnd; ++iterH) {
			iterW = (*iterH).begin();
			wEnd = (*iterH).end();
			for (; iterW != wEnd; ++iterW) {
				if ((*iterW) != 0) {
					return false;
				}
			}
		}

		return true;

	} else {
		return true;	// empty matrix is considered to be null
	}	// if (!argM.empty())
}

// Calculates average value in given array
double qavg (__VECTOR_DOUBLE& argV) {
	int size = (int)argV.size();
	return qsum(argV)/size;
}

double qavg (__VECTOR_INT& argV) {
	int size = (int)argV.size();
	return qsum(argV)/(double)size;
}

double qsum (__VECTOR_DOUBLE& argV) {
	int i, size = (int)argV.size();
	double sum=0;

	for (i=0; i<size; i++) {
		sum += argV[i];
	}
	return sum;
}

int qsum (__VECTOR_INT& argV) {
	int i, size = (int)argV.size();
	int sum=0;

	for (i=0; i<size; i++) {
		sum += argV[i];
	}
	return sum;
}

void qstats (__VECTOR_DOUBLE& argV, double& mean, double& stdvar) {
	double size_1 = 1.0 / (double)argV.size(), size = (double)argV.size();
	__VECTOR_DOUBLE::iterator iter = argV.begin();

	mean = stdvar = 0;
	for (; iter != argV.end(); iter++) {
		mean += (*iter);
		stdvar += (*iter) * (*iter);
	}
	mean *= size_1;
	stdvar = stdvar*size_1 - mean*mean;
	stdvar *= size/(size-1);
}

// Calculates mean and variance (not deviation!) of the given array
void qstats (__VECTOR_INT& argV, double& mean, double& stdvar) {
	double size_1 = 1.0 / (double)argV.size(), size = (double)argV.size();
	__VECTOR_INT::iterator iter = argV.begin();

	mean = stdvar = 0;
	for (; iter != argV.end(); iter++) {
		mean += (*iter);
		stdvar += (*iter) * (*iter);
	}
	mean *= size_1;
	stdvar = stdvar*size_1 - mean*mean;
	stdvar *= size/(size-1);
}

// Creates histogram from given frequency table
// Returns normalization coefficient
double qgethistogram (const __MAP_DOUBLE_INT& argS, __MAP_DOUBLE_INT& argT, const int argN) {
	int i;//, size = (int)argS.size();
	double max, min, step;
	double norm;
	double temp;
	__MAP_DOUBLE_INT::const_iterator iter;

	// find maximum and minimum elements in table
	max = -1000000;
	min = 1000000;
	norm = 0;
	for (iter=argS.begin(); iter!=argS.end(); iter++) {
		temp = (*iter).first;
		if (temp > max) {
			max = temp;
		}
		if (temp < min) {
			min = temp;
		}
		norm += (*iter).second;
	}
	step = (max - min) / argN;	// width of each category
//	norm = 1.0 / norm;	// normalization coefficient

	// initialize target array
	argT.clear ();
	temp = min;
	for (i=0; i < argN; i++, temp+=step) {
		argT.insert (__MAP_DOUBLE_INT::value_type (temp, 0));
	}

	// fill target array
	__MAP_DOUBLE_INT::iterator titer;
	for (iter=argS.begin(); iter!=argS.end(); iter++) {
		titer = argT.lower_bound ((*iter).first - step);
		(*titer).second += ((*iter).second);
	}

	return norm;
}

double logsigmoid (const int size, const double* inputs, const double* weights, const double aspect, const double offset) {
	int i;
	double res=offset;

	for (i=0; i<size; i++) {
		res += inputs[i] * weights[i];
	}
	res *= aspect;
	return 1.0/(1.0+exp(-res));
}

// Copies one char array into another. Source char array should be ended
//	with null-terminator '/0'.
int qstrcpy (char*& t, const char* s) {
/*	cout<<"qstrcpy. source = "<<s<<"\n";
	cout.flush ();/**/
	int l=0;
	if (t != NULL) delete[] t;	// delete possible old contents of target array
	while (s[l++] != '\0') {}	// define source array length
	t = new char [l];		// create new array of chars
	l--;
	for (; l>=0; l--) {
		t[l] = s[l];		// copy chars from source to target
	}
/*	cout<<"target = "<<t<<endl;/**/
	return 0;
}

// Copies one char array into another and nulls source array.
//	Chars in source array should be ended with null-terminator '/0'.
int qstrcpy0 (char* t, char* s) {
	int l=0;
	if (t != NULL) delete[] t;	// delete possible old contents of target array
	while (s[l++] != '\0') {}	// define source array length
	t = new char [l];		// create new array of chars
	l--;
	for (; l>=0; l--) {
		t[l] = s[l];		// copy chars from source to target
		s[l] = 0;					// null source array
	}
	return 0;
}

int trim (string& str) {
	int pos;
	while ((pos = (int)str.find_first_of ('\n')) >= 0) {	// while there CR symbols
		str.erase (pos, 1);
	}
	// trim string's head
	while (str[0] == '\t' || str[0] == ' ') {
		str.erase (0, 1);
	}
	// trim string's tail
	if (!str.empty()) {
		while (str[ str.length()-1 ] == '\t' || str[ str.length()-1 ] == ' ') {
			str.erase (str.length()-1, 1);
		}
	}

	return Q_OK;
}

////////////////////////////////////////////////////////////////////////////
// Converts representation of the integer from string to the integer type
int char2int (int& t, const char* s) {
	int order=10, i=0, temp;

	t = 0;
	if (s[0] == '-') {
		i = 1;
		temp = -1;
	} else {
		temp = 1;
	}
	for (; s[i]!=0; i++) {
		t += s[i] - 0x30;
		t *= order;
	}
	t /= order;
	t *= temp;

	return t;
}

////////////////////////////////////////////////////////////////////////////
// Converts representation of the number from string to the double type
double char2dbl (double& t, const char* s) {
	int i=0, temp, sign;
	double order=1;

	for (i=0; s[i]!='\0'; i++);	// calculate length of source string
	t = 0;
	if (s[0] == '-') {
		sign = 1;
	}	else {
		sign = 0;
	}
	for (i--; i>=sign; i--, order*=10) {
		if (s[i]>='0' && s[i]<='9') {
			temp = s[i] - 0x30;
			t += temp*order;
		} else if (s[i] == '.') {
			t /= order;
			order = 0.1;
		}
	}
	if (sign) {t *= -1;}

	return t;
}

////////////////////////////////////////////////////////////////////////////
// Converts representation of the number from string to the double type
int dbl2char (char*& t, const double s) {
	if (s == 0) {
		t = new char[2];
		t[0] = '0';
		t[1] = '\0';
		return 0;
	}

	int i=0, j, iTemp = (int)fabs(s), pre, past;
	double dTemp = fabs(s) - iTemp;
	char after[8], before[32];

	// define number of digits after comma
	while (dTemp != 0 && i<8) {
		dTemp *= 10;
		iTemp = (int)dTemp;
		dTemp -= iTemp;
		after[i] = (char)(iTemp + 0x30);
		i++;
	}
	past = i;

	// define number of digits before comma
	i=0;
	iTemp = (int)fabs(s);
	while (iTemp != 0) {
		pre = iTemp % 10;
		before[31 - i] = (char)(pre + 0x30);
		iTemp = (int)(iTemp * 0.1);
		i++;
	}
	pre = i;
	if (s > 0) {
		t = new char[pre + 1 + past + 1];
		j = 0;
	} else {	// s < 0
		t = new char[1 + pre + 1 + past + 1];	// additional char for sign
		j = 1;
		t[0] = '-';
	}
	for (; pre>0; pre--, j++) {
		t[j] = before[32-pre];
	}
	if (past != 0) {
		t[j++] = '.';
		for (i=0; i<past; i++, j++) {
			t[j] = after[i];
		}
	}
	t[j] = 0;
//	cout<<"\tt = "<<t<<endl;
	return 0;
}

int hex2dec (int& target, const char* source, const int sourceSize) {
	int i;

	target = 0;
	for (i = sourceSize-1; i >= 0; i--) {
		target += ((unsigned char)(source[i])) << (8 * i);
	}

	return target;
}

////////////////////////////////////////////////////////////////////////////
// Reads input and output sets of data from text file and put it into
//   corresponding 'ins' and 'outs' objects
// First string in file should mark columns Xi and Yi, where Xi matches inputs
//   and Yi matches outputs. There should be NO NEW LINE(S) at the end of file.
//   Example (XOR):
//    X1 X2 Y1
//    0  0  0
//    0  1  1
//    1  0  1
//    1  1  0
int read_ann_data (const char* filename, __TABLE_DOUBLE& ins, __TABLE_DOUBLE& outs) {
	int i, j;
	int InputsNumber=0, OutputsNumber=0;
	char c=0;
	__VECTOR_DOUBLE tempI, tempO, tempV;
	ifstream fin(filename);

	// look for input and output variables and count them
	while (c != '\n') {
		c = (char)fin.get ();
		if (c == 'X') {
			InputsNumber++;
		} else if (c == 'Y') {
			OutputsNumber++;
		}
	}
/*	cout<<"Inputs number = "<<InputsNumber<<endl
			<<"Outputs number = "<<OutputsNumber<<endl;/**/
	tempI.resize (InputsNumber);
	tempO.resize (OutputsNumber);

	j=0;
	while (fin.eof() == 0) {	// while not end of data file
		for (i=0; i<InputsNumber; i++) {
			fin>>tempI[i];
		}
		for (i=0; i<OutputsNumber; i++) {
			fin>>tempO[i];
		}
		//ins.Add (InputsNumber, tempI);
		//outs.Add (OutputsNumber, tempO);
		ins.push_back (tempI);
		outs.push_back (tempO);
		// look for the end of line
		c = (char)fin.get ();
		while (c!='\n' && c!=EOF) {
			c = (char)fin.get ();
		}
		j++;
	}
//	cout<<"Added "<<j<<" sets\n";

	fin.close ();
	tempI.clear ();
	tempO.clear ();
	//if (tempI != NULL) delete[] tempI;
	//if (tempO != NULL) delete[] tempO;
	return 0;
}

int readBMP (const char *argFilename, QRGB**& argImage, int& argWidth, int& argHeight) {
//	ifstream fin;
	FILE* fin;
//	QRGB** tempImg;
	QBMPHeader header;
	QBMPInfoHeader infoHeader;
	int i;//, j;
//	int size, offset, width, height, planes, bitCount, compression, sizeImage, xpels, ypels, colorsUsed, colorsMain;
	int bitCount;
	QRGB black;
	char tempChar[3];

	black.r = black.g = black.b = 0;

//	fin.open (argFilename, ios_base::in | ios_base::binary);
//	cout<<"Going out!\n";
	fin = fopen (argFilename, "rb");
//	fopen_s (&fin, argFilename, "rb");
	if (fin != NULL) {
//		ofstream fout;
//		fout.open ("test.log");

		i = (int)fread (&header.signature, 1, 2, fin);
		i = (int)fread (&header.size, 1, 4, fin);
		i = (int)fread (&header.res1, 1, 2, fin);
		i = (int)fread (&header.res2, 1, 2, fin);
		i = (int)fread (&header.offset, 1, 4, fin);
//		cout<<"Read "<<i<<" characters\n";
		i = (int)fread (&infoHeader.size, sizeof(char), sizeof(infoHeader), fin);

		//hex2dec (size, header.size, sizeof (header.size));
		//hex2dec (offset, header.offset, sizeof (header.offset));
 
		cout<<"File header ("<<argFilename<<"):\n"
			<<"\tSignature: "<<header.signature<<"\n"
			<<"\tSize: "<<header.size<<"\n"
			<<"\tOffset: "<<header.offset<<"\n";

		cout<<"\nBMP info header:\n"
			<<"\tSize: "<<infoHeader.size<<"\n"
			<<"\tWidth: "<<infoHeader.width<<"\n"
			<<"\tHeight: "<<infoHeader.height<<"\n"
			<<"\tPlanes: "<<infoHeader.planes<<"\n"
			<<"\tBitCount: "<<infoHeader.bitsPerPixel<<"\n"
			<<"\tCompression: "<<infoHeader.compression<<"\n"
			<<"\tSizeImage: "<<infoHeader.sizeImage<<"\n"
			<<"\tXpels: "<<infoHeader.pixelsPerMeterX<<"\n"
			<<"\tYpels: "<<infoHeader.pixelsPerMeterY<<"\n"
			<<"\tColorsUsed: "<<infoHeader.colorsUsed<<"\n"
			<<"\tColorsMain: "<<infoHeader.colorsMain<<"\n"<<dec;

		bitCount = infoHeader.bitsPerPixel;
		if (infoHeader.compression == 0) {
			if (bitCount == 1) {
			} else if (bitCount == 4) {
			} else if (bitCount == 8) {
				//
				// 256-colors image
				//
//				QBMPColorTable colorTable;

				// read palette
				// read image
			} else if (bitCount == 24) {
				//
				// True color image
				//
				//fpos_t pos;
				//fgetpos(fin, &pos);
				//cout<<pos<<"\n";

				argHeight = infoHeader.height;
				argWidth = infoHeader.width;
//				tempImg = new QRGB* [height];
				argImage = new QRGB* [argHeight];
				// create image pixels array and fill it with 0s
				for (i=0; i<argHeight; i++) {
//					tempImg[i] = new QRGB [width];
//					memset (tempImg[i], 0, width*sizeof(QRGB));	// set to white (or to any other color)
					argImage[i] = new QRGB [argWidth];
					memset (argImage[i], 0, argWidth*sizeof(QRGB));	// set to white (or to any other color)
				}

				int counter=0;	//, temp;
//				fout<<"Starting to read image... "<<fin.tellg ()<<"\n";
				for (i=(argHeight-1); i >= 0; i--) {
					counter += argWidth;
					fread (argImage[i], argWidth*sizeof(QRGB), sizeof(char), fin);
//					fread (tempImg[i], sizeof(char), width*sizeof(QRGB), fin);
					fread (tempChar, sizeof(char), (argWidth%4), fin);
					//fgetpos(fin, &pos);
					//cout<<pos<<"\n";
				}
				//cout<<"Read RGBs: "<<counter<<"\n";
			}
		} else {
			cout<<"readBMP: Can't read compressed BMP files\n";
		}	// end of if (compression == 0)
//		fin.close ();
		fclose (fin);
//		argImage = tempImg;

		//ofstream fout;
		//fout.open ("test.log");
		//for (i=0; i<argHeight; i++) {
		//	for (j=0; j<argWidth; j++) {
		//		fout<<argImage[i][j].r<<argImage[i][j].g<<argImage[i][j].b;
		//	}
		//	fout<<"\n";
		//}
		//fout.close ();

		return Q_OK;

	} else {
		cout<<"readBMP: Can't open file: "<<argFilename<<"\n";
		return Q_FAILURE;
	}	// end of if (fin.is_open ())
}

int writeBMP (const char *argFilename, QRGB** const argImage, const int argWidth, const int argHeight) {
	cout<<"writeBMP\n";
	cout.flush ();
//	ofstream fout;
	FILE* fout;
	QBMPHeader header;
	QBMPInfoHeader infoHeader;
	char temp[3];
	int i, j;

//	fout.open (argFilename, ios_base::out | ios_base::binary);
	fout = fopen (argFilename, "wb");
//	fopen_s (&fout, argFilename, "wb");
	if (fout != NULL) {
		header.signature = 0x4D42;	// 'BM'
		header.offset = 54;
		header.size = 14 + 40 + argWidth*argHeight*3 + argHeight*((3*argWidth)%4);
		header.res1 = header.res2 = 0;

		fwrite (&header.signature, sizeof(char), 2, fout);
		fwrite (&header.size, sizeof(char), 4, fout);
		fwrite (&header.res1, sizeof(char), 2, fout);
		fwrite (&header.res2, sizeof(char), 2, fout);
		fwrite (&header.offset, sizeof(char), 4, fout);

		infoHeader.size = 40;
		infoHeader.width = argWidth;
		infoHeader.height = argHeight;
		infoHeader.planes = 1;
		infoHeader.bitsPerPixel = 24;
		infoHeader.compression = 0;
		infoHeader.sizeImage = 0;
		infoHeader.colorsMain = infoHeader.colorsUsed = 0;
		infoHeader.pixelsPerMeterX = infoHeader.pixelsPerMeterY = 0;
		fwrite (&infoHeader, sizeof(char), 40, fout);

		temp[0] = temp[1] = temp[2] = 0;
		for (i=(argHeight-1); i>=0; i--) {
//			cout<<i<<"\n";
			j = (int)fwrite (argImage[i], sizeof(QRGB), argWidth, fout);
//			cout<<"written "<<j<<" chars\n";
//			cout.flush ();
			fwrite (temp, sizeof(char), (argWidth%4), fout);
		}

		fclose (fout);

		return Q_OK;

	} else {
		cout<<"writeBMP: Can't open file \""<<argFilename<<"\"\n";
		return Q_FAILURE;
	}	// if (fout.is_open ())
}

int qhisteq (QRGB** argImage, const int argWidth, const int argHeight) {
	int i, j, total = argWidth * argHeight;
	__VECTOR_DOUBLE pr, pg, pb;	// source and target brightness distributions respectively
	//ofstream fout;
	//fout.open ("imglog.log");

	// calculate vector px contents
	pr.resize (256);
	pr.assign (256, 0);
	pg.resize (256);
	pg.assign (256, 0);
	pb.resize (256);
	pb.assign (256, 0);
	for (i=0; i<argHeight; i++) {
		for (j=0; j<argWidth; j++) {
			pr[ argImage[i][j].r ] ++;
			pg[ argImage[i][j].g ] ++;
			pb[ argImage[i][j].b ] ++;
		}
	}
	// normalization
//	fout<<"Source proportion (r : g : b):\n";
	for (i=0; i<256; i++) {
		pr[i] *= 255.0/(double)total;
		pg[i] *= 255.0/(double)total;
		pb[i] *= 255.0/(double)total;
//		fout<<i<<":\t"<<pr[i]<<"\t"<<pg[i]<<"\t"<<pb[i]<<"\t\n";
	}

	// define new brightness levels
//	fout<<"\nTransition functions for each color plane (r : g : b):\n";
//	fout<<"0:\t"<<sr[0]<<"\t"<<sg[0]<<"\t"<<sb[0]<<"\t\n";
	for (i=1; i<256; i++) {
		//sr[i] = sr[i-1] + pr[i];
		//sg[i] = sg[i-1] + pg[i];
		//sb[i] = sb[i-1] + pb[i];
		//fout<<i<<":\t"<<sr[i]<<"\t"<<sg[i]<<"\t"<<sb[i]<<"\t\n";
		pr[i] += pr[i-1];
		pg[i] += pg[i-1];
		pb[i] += pb[i-1];
//		fout<<i<<":\t"<<pr[i]<<"\t"<<pg[i]<<"\t"<<pb[i]<<"\t\n";
	}
//	fout.close ();

	// redefine brightness levels
	for (i=0; i<argHeight; i++) {
		for (j=0; j<argWidth; j++) {
			//argImage[i][j].r = (__QBYTE)(sr[ (int)argImage[i][j].r ] + 0.5);
			//argImage[i][j].g = (__QBYTE)(sg[ (int)argImage[i][j].g ] + 0.5);
			//argImage[i][j].b = (__QBYTE)(sb[ (int)argImage[i][j].b ] + 0.5);
			argImage[i][j].r = (__QBYTE)(pr[ (int)argImage[i][j].r ] + 0.5);
			argImage[i][j].g = (__QBYTE)(pg[ (int)argImage[i][j].g ] + 0.5);
			argImage[i][j].b = (__QBYTE)(pb[ (int)argImage[i][j].b ] + 0.5);
		}
	}

	return Q_OK;
}

int edgeSobel (QImage& argImage, const double argT) {
	if (argImage.getWidth() != 0 && argImage.getHeight() != 0) {

		QImage image;
		int i, j;
		int width = argImage.getWidth(), height = argImage.getHeight();
		double d[2];
		int av1, av2;

		// Apply Sobel's transformation (see K.Lindley for details)
		image = argImage;
		for (i=1; i<(height-1); i++) {
			for (j=1; j<(width-1); j++) {
				av1 = argImage[i-1][j].r + argImage[i-1][j+1].r + argImage[i][j+1].r;	// sum of b-c-f
				av2 = argImage[i+1][j].r + argImage[i+1][j-1].r + argImage[i][j-1].r;	// sum of d-g-h
				d[0] = abs (av1 - av2);	// for a-e-i

				av1 = argImage[i-1][j-1].r + argImage[i][j-1].r + argImage[i+1][j-1].r;	// sum of a-d-g
				av2 = argImage[i+1][j+1].r + argImage[i][j+1].r + argImage[i-1][j+1].r;	// sum of c-f-i
				d[1] = abs (av1 - av2);	// for b-e-h
				if (d[1] > d[0]) {
					d[0] = d[1];
				}

				av1 = argImage[i-1][j-1].r + argImage[i-1][j].r + argImage[i][j-1].r;	// sum of a-b-d
				av2 = argImage[i+1][j+1].r + argImage[i+1][j].r + argImage[i][j+1].r;	// sum of f-h-i
				d[1] = abs (av1 - av2);	// for c-e-g
				if (d[1] > d[0]) {
					d[0] = d[1];
				}

				av1 = argImage[i-1][j-1].r + argImage[i-1][j].r + argImage[i-1][j+1].r;	// sum of b-c-f
				av2 = argImage[i+1][j+1].r + argImage[i+1][j].r + argImage[i+1][j-1].r;	// sum of d-g-h
				d[1] = abs (av1 - av2);	// for d-e-f
				if (d[1] > d[0]) {
					d[0] = d[1];
				}

				d[0] = d[0]/3.0 + 0.5;	// normalization and preparation for proper rounding
				if (d[0] < argT) {	//0.618 * argImage.getAverageLuma())) {
//				if (d[0] < 128) {
					image[i][j].b = image[i][j].g = image[i][j].r = 0;
				} else {
					image[i][j].b = image[i][j].g = image[i][j].r = (__QBYTE)(d[0]);
				}
			}
		}
		argImage = image;
		return Q_OK;

	} else {
		cout<<"edgeSobel:\tCan't transform empty image\n";
		return Q_FAILURE;
	}	// if (argImage.getWidth() != 0 && argImage.getHeight() != 0)
}

int rgb2Grayscale (const QImage* source, QImage* target) {
	int i, j;

	if (source != NULL) {
		target->setDimensions (source->getWidth(), source->getHeight());
		// convert source rgb-image to the grayscale target image
		for (i=0; i<source->getHeight(); i++) {
			for (j=0; j<source->getWidth(); j++) {
			}
		}
	} else {
		// convert target rgb-image to the grayscale (source and target is one and the same image)
	}
	return 1;
}

double calculatePSNR (const __TABLE_DOUBLE& argRef, const __TABLE_DOUBLE& argT) {
	int i, j;
	int width, height;
	double temp, psnr;

	height = (int)argRef.size ();
	width = (int)argRef[0].size ();
	temp = 0.0;
	for (i=0; i<height; i++) {
		for (j=0; j<width; j++) {
			temp += (argRef[i][j] - argT[i][j])*(argRef[i][j] - argT[i][j]);
		}
	}
	temp /= (double)(height * width);
	psnr = 10.0 * log10(1.0 / temp);

	return psnr;
}

double evaluateImage (const QImage * argImage) {
	double eval = 0.0;

	int width, height, square;
	int i, j;
	__VECTOR_DOUBLE greyFreq;	// table of frequencies for grey levels
	double avLuma;	// average brightness level after transformation
	double	_H,	// entropy of transformed images
			_Mu;	// number of edgels of transformed images
	double log2_1 = 1.0 / log(2.0);
	double delim_255 = 1.0 / 128.0;
	QImage tempImage;

	//MessageBox (NULL, "Validation Complete", "QANNImageTask::evaluateSolution (void* argSolution)", MB_OK);
	// prepare auxiliary containers
	greyFreq.resize (256);
	width = argImage->getWidth ();
	height = argImage->getHeight ();
	square = width * height;
	avLuma = argImage->getMeanBrightness();
	greyFreq.assign (256, 0);

	// calculate grey frequencies
	for (i=0; i<height; i++) {
		for (j=0; j<width; j++) {
			// calculate frequences for each level of grey in transformed picture
			greyFreq[ argImage->getPixel(i,j)->g ]++;
		}
	}
	_H = 0;
	for (i=0; i<256; i++) {
		if (greyFreq[i] != 0) {
			greyFreq[i] /= (double)square;
			_H -= greyFreq[i] * log(greyFreq[i]);// * log2_1;
		}
	}
	tempImage = (*argImage);
	tempImage.edgeSobel ();
	_Mu = tempImage.calculatePixels (avLuma);
	eval += (1 + 255 - exp(_H))/192.0 + ((double)square - _Mu) / ((double)square);

	return eval;
}

double evaluateImageMod (const QImage * argImage) {
	double eval = 0.0;

	int width, height, square;
	int i, j;
	__VECTOR_DOUBLE greyFreq;	// table of frequencies for grey levels
	double avLuma;	// average brightness level after transformation
	double	_H,	// entropy of transformed images
			_E,	// mean intensity of edgels
			_Mu;	// number of edgels of transformed images
	double log2_1 = 1.0 / log(2.0);
	double delim_255 = 1.0 / 128.0;
	QImage tempImage;

	//MessageBox (NULL, "Validation Complete", "QANNImageTask::evaluateSolution (void* argSolution)", MB_OK);
	// prepare auxiliary containers
	greyFreq.resize (256);
	width = argImage->getWidth ();
	height = argImage->getHeight ();
	square = width * height;
	avLuma = argImage->getMeanBrightness();
	greyFreq.assign (256, 0);

	// calculate grey frequencies
	for (i=0; i<height; i++) {
		for (j=0; j<width; j++) {
			// calculate frequences for each level of grey in transformed picture
			greyFreq[ argImage->getPixel(i,j)->g ]++;
		}
	}
	_H = 0;
	for (i=0; i<256; i++) {
		if (greyFreq[i] != 0) {
			greyFreq[i] /= (double)square;
			_H -= greyFreq[i] * log(greyFreq[i]);// * log2_1;
		}
	}
	tempImage = (*argImage);
	tempImage.edgeSobel ();
	_E = 0.0;
	for (i=0; i<height; i++) {
		for (j=0; j<width; j++) {
			_E += tempImage[i][j].g;
		}
	}
	_Mu = tempImage.calculatePixels (avLuma);
	_E /= tempImage.calculatePixels (0.0);
	eval = (256.0 - exp(_H))/192.0 + ((double)square - _Mu) / ((double)square) + (255.0 - _E)/255.0;

	return eval;
}

double evaluateImageMunteanuRosa (const QImage * argImage) {
	double eval = 0.0;

	int width, height, square;
	int i, j;
	__VECTOR_DOUBLE greyFreq;	// table of frequencies for grey levels
	double avLuma;	// average brightness level after transformation
	double	_H,	// entropy of transformed images
			_E,	// intensity of the edge pixels
			_Mu;	// number of edgels of transformed images
	double log2_1 = 1.0 / log(2.0);
	double delim_255 = 1.0 / 128.0;
	QImage tempImage;

	//MessageBox (NULL, "Validation Complete", "QANNImageTask::evaluateSolution (void* argSolution)", MB_OK);
	// prepare auxiliary containers
	greyFreq.resize (256);
	width = argImage->getWidth ();
	height = argImage->getHeight ();
	square = width * height;
	avLuma = argImage->getMeanBrightness();
	greyFreq.assign (256, 0);

	// calculate grey frequencies
	for (i=0; i<height; i++) {
		for (j=0; j<width; j++) {
			// calculate frequences for each level of grey in transformed picture
			greyFreq[ argImage->getPixel(i,j)->g ]++;
		}
	}
	_H = 0;
	for (i=0; i<256; i++) {
		if (greyFreq[i] != 0) {
			greyFreq[i] /= (double)square;
			_H -= greyFreq[i] * log(greyFreq[i]);// * log2_1;
		}
	}
	tempImage = (*argImage);
	tempImage.edgeSobel ();
	_E = 0;
	for (i=0; i<height; i++) {
		for (j=0; j<width; j++) {
			_E += tempImage[i][j].g;
		}
	}
	_Mu = tempImage.calculatePixels (avLuma);
	eval = log(log(_E + 2.718281828))*exp(_H)*_Mu/(double)square;

	return eval;
}

////////////////////////////////////////////////////////////////////////////
//
// Miscellaneous classes definition
//

////////////////////////////////////////////////////////////////////////////
// QImage class definition
//
QImage::QImage () : _image(NULL), _width(0), _height(0),
					_avR(0), _avG(0), _avB(0), _avBrightness(0),
					_maxR(0), _maxG(0), _maxB(0),
					_minR(0), _minG(0), _minB(0),
					ignoreOutliers(false) {
}

QImage::QImage (const QImage& argImage) {
	int i;

	_height = argImage._height;
	_width = argImage._width;
	_avR = argImage._avR;
	_avG = argImage._avG;
	_avB = argImage._avB;
	_avBrightness = argImage._avBrightness;
	_image = new QRGB* [_height];
	for (i=0; i<_height; i++) {
		_image[i] = new QRGB [_width];
		memcpy (_image[i], argImage._image[i], _width * sizeof(QRGB));
	}
}

QImage::~QImage () {
	eraseImage ();
}

void QImage::eraseImage () {
	if (_image != NULL) {
		for (int i=0; i<_height; i++) {
			delete[] _image[i];
		}
		delete[] _image;
	}
}

QRGB* QImage::operator [] (const int index) {
	return _image[index];
}

QRGB* QImage::operator [] (const int index) const {
	return _image[index];
}

QImage& QImage::operator = (const QImage& source) {
	int i;

	if (_image != NULL) {
		for (i=0; i<_height; i++) {
			delete[] _image[i];
		}
		delete[] _image;
	}

	_height = source._height;
	_width = source._width;
	_avR = source._avR;
	_avG = source._avG;
	_avB = source._avB;
	_avBrightness = source._avBrightness;
	_image = new QRGB* [_height];
	for (i=0; i<_height; i++) {
		_image[i] = new QRGB [_width];
		memcpy (_image[i], source._image[i], _width * sizeof(QRGB));
	}

	return (*this);
}

bool QImage::operator == (const QImage& argImage) {
	// check dimensions
	if (_width != argImage.getWidth() || _height != argImage.getHeight()) {
		return false;
	}

	int i, j;

	// check pixels
	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			if (_image[i][j].r != argImage.getPixel (i, j)->r ||
				_image[i][j].g != argImage.getPixel (i, j)->g ||
				_image[i][j].b != argImage.getPixel (i, j)->b) {
					return false;
			}
		}
	}

	return true;
}

int QImage::toGrayscale () {
	if (_image != NULL) {
		// Convert image using formula:
		// Y = 0.212671 * R + 0.715160 * G + 0.072169 * B;
		int i, j;
		QRGB* tempRGB;
		double temp;

		tempRGB = new QRGB [ _width ];
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				temp = 0.212671 * _image[i][j].r + 0.715160 * _image[i][j].g + 0.072169 * _image[i][j].b;
				temp = (temp + 0.5);
				tempRGB[j].r = tempRGB[j].g = tempRGB[j].b = (__QBYTE)temp;
			}
			memcpy (_image[i], tempRGB, _width * sizeof(QRGB));
		}
		delete[] tempRGB;

		//_avLuma = 0;
		//for (i=0; i<_height; i++) {
		//	for (j=0; j<_width; j++) {
		//		_avLuma += _image[i][j].r;
		//	}
		//}
		//_avLuma /= (_width * _height);
		//cout<<"_avLuma = "<<_avLuma<<"\n";
		recalculateStatistics ();

		return Q_OK;

	} else {

		return Q_FAILURE;

	}
}

int QImage::toGrayscaleAvg () {
	if (_image != NULL) {
		// Convert image using simple averaging formula:
		// Y = 0.333333 * (R + G + B);
		int i, j;
		QRGB* tempRGB;
		double temp;

		tempRGB = new QRGB [ _width ];
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				temp = 0.333333 * (_image[i][j].r + _image[i][j].g + _image[i][j].b);
				temp = (temp + 0.5);
				tempRGB[j].r = tempRGB[j].g = tempRGB[j].b = (__QBYTE)temp;
			}
			memcpy (_image[i], tempRGB, _width * sizeof(QRGB));
		}
		delete[] tempRGB;

		recalculateStatistics ();

		return Q_OK;

	} else {

		return Q_FAILURE;

	}
}

int QImage::toGrayscaleNTSC () {
	if (_image != NULL) {
		// Convert image using formula:
		// Y = 0.299 * R + 0.587 * G + 0.114 * B;
		int i, j;
		QRGB* tempRGB;
		double temp;

		tempRGB = new QRGB [ _width ];
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				temp = 0.299 * _image[i][j].r + 0.587 * _image[i][j].g + 0.114 * _image[i][j].b;
				temp = (temp + 0.5);
				tempRGB[j].r = tempRGB[j].g = tempRGB[j].b = (__QBYTE)temp;
			}
			memcpy (_image[i], tempRGB, _width * sizeof(QRGB));
		}
		delete[] tempRGB;

		_avBrightness = 0;
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				_avBrightness += _image[i][j].r;
			}
		}
		_avBrightness /= (_width * _height);
		cout<<"_avBrightness = "<<_avBrightness<<"\n";

		return Q_OK;

	} else {

		return Q_FAILURE;

	}
}

int QImage::edgeSobel () {
	int ret = ::edgeSobel (*this, 0);
	return ret;
}

int QImage::calculatePixels (const double argLuma) {
	int count=0;
	int i, j;

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			if (_image[i][j].b > argLuma) {
				count++;
			}
		}
	}

	return count;
}

double QImage::calculateSumBrightness (double argL) {
	double sum=0;
	int i, j;

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			if (_image[i][j].b > argL) {
				sum += _image[i][j].b;
			}
		}
	}

	return sum;
}


int QImage::recalculateStatistics () {
	int i, j;
//	__QBYTE r, g, b;
	unsigned char temp;
	double avR2, avG2, avB2;

	_avR = _avG = _avB = 0;
	avR2 = avG2 = avB2 = 0;
	_maxR = _maxG = _maxB = 0;
	_minR = _minG = _minB = 255;
	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			temp = _image[i][j].r;
			_avR += temp;
			avR2 += temp*temp;
			if (temp > _maxR) {_maxR = temp;}
			if (temp < _minR) {_minR = temp;}

			temp = _image[i][j].g;
			_avG += temp;
			avG2 += temp*temp;
			if (temp > _maxG) {_maxG = temp;}
			if (temp < _minG) {_minG = temp;}

			temp = _image[i][j].b;
			_avB += temp;
			avB2 += temp*temp;
			if (temp > _maxB) {_maxB = temp;}
			if (temp < _minB) {_minB = temp;}
		}
	}
	_avR /= (_width * _height);
	_avG /= (_width * _height);
	_avB /= (_width * _height);
	_avBrightness = 0.212671*_avR + 0.715160*_avG + 0.072169*_avB;

	avR2 /= (_width * _height);
	avG2 /= (_width * _height);
	avB2 /= (_width * _height);
	_devBrightness = 0.212671*avR2 + 0.715160*avG2 + 0.072169*avB2;
	_devBrightness -= _avBrightness*_avBrightness;

	return Q_OK;
}

int QImage::readFromFile (const char* filename) {
	int i;//, j;

	// if there is an image loaded
	// then it souhld be deleted
	if (_width*_height != 0) {
		for (i=0; i<_height; i++) {
			delete[] _image[i];
		}
		delete[] _image;
		cout<<"Previous image deleted.\n";
		cout.flush ();
	}

	int ret = readBMP (filename, _image, _width, _height);

	recalculateStatistics ();

	cout<<"_minR = "<<(int)_minR<<"\n";
	cout<<"_maxR = "<<(int)_maxR<<"\n";
	cout<<"_avR = "<<_avR<<"\n\n";
	cout<<"_minG = "<<(int)_minG<<"\n";
	cout<<"_maxG = "<<(int)_maxG<<"\n";
	cout<<"_avG = "<<_avG<<"\n\n";
	cout<<"_minB = "<<(int)_minB<<"\n";
	cout<<"_maxB = "<<(int)_maxB<<"\n";
	cout<<"_avB = "<<_avB<<"\n\n";

	return ret;
}

int QImage::writeToFile (const char* filename) {
	int ret = writeBMP (filename, _image, _width, _height);
	return ret;
}

int QImage::writeToFile (const char* filename) const {
	int ret = writeBMP (filename, _image, _width, _height);
	return ret;
}

int QImage::toColor (const QImage& argImage) {
	int i, j;
	QImage tempImage (argImage);
	double aspect;
//	double alpha, beta, gamma;
//	__QBYTE tempR, tempG, tempB;
	double newR, newG, newB;
	double aspectR = 0.8, aspectG = 1.1, aspectB = 1.1;

	tempImage.toGrayscale ();	// convert to get the luminance map
	recalculateStatistics ();
	aspect = _avBrightness / tempImage.getMeanBrightness ();

	__QBYTE maxRGB;	// (maximum+1) value of the RGB-component in order not to exceed 255 value

	if (aspect > 1.0) {
		maxRGB = (__QBYTE)(255.0 / aspect + 1.0);
	} else {
		maxRGB = 255;
	}

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			if (tempImage[i][j].g != 0) {
				aspect = (double)_image[i][j].g / (double)tempImage[i][j].g;
			} else {	// if tempImage[i][j] is black than paint target pixel black too
				aspect = 0;
			}

			newR = ((double)argImage.getPixel (i, j)->r * aspect * aspectR);
			newG = ((double)argImage.getPixel (i, j)->g * aspect * aspectG);
			newB = ((double)argImage.getPixel (i, j)->b * aspect * aspectB);

			if (newR < 255.0) {
				_image[i][j].r = (__QBYTE)newR;
			} else {
				_image[i][j].r = 255;
			}

			if (newG < 255.0) {
				_image[i][j].g = (__QBYTE)newG;
			} else {
				_image[i][j].g = 255;
			}

			if (newB < 255.0) {
				_image[i][j].b = (__QBYTE)newB;
			} else {
				_image[i][j].b = 255;
			}

		}
	}
	recalculateStatistics ();

	return Q_OK;
}

int QImage::fillWhite () {
	int i, j;
	QRGB temp;

	temp.r = temp.g = temp.b = 255;

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			_image[i][j] = temp;
		}
	}

	return Q_OK;
}

int QImage::changeLightnessMult (const double argAspect) {
	if (argAspect == 1.0) {
		return Q_OK;
	}

	int i, j;
	__QBYTE maxRGB;	// (maximum+1) value of the RGB-component in order not to exceed 255 value

	if (argAspect > 1.0) {
		maxRGB = (__QBYTE)(255.0 / argAspect + 1.0);
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				if (_image[i][j].r < maxRGB) {
					_image[i][j].r = (__QBYTE)((double)_image[i][j].r * argAspect + 0.5);
				} else {
					_image[i][j].r = 255;
				}
				if (_image[i][j].g < maxRGB) {
					_image[i][j].g = (__QBYTE)((double)_image[i][j].g * argAspect + 0.5);
				} else {
					_image[i][j].g = 255;
				}
				if (_image[i][j].b < maxRGB) {
					_image[i][j].b = (__QBYTE)((double)_image[i][j].b * argAspect + 0.5);
				} else {
					_image[i][j].b = 255;
				}
			}
		}
	} else {
		maxRGB = 255;
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				_image[i][j].r = (__QBYTE)((double)_image[i][j].r * argAspect + 0.5);
				_image[i][j].g = (__QBYTE)((double)_image[i][j].g * argAspect + 0.5);
				_image[i][j].b = (__QBYTE)((double)_image[i][j].b * argAspect + 0.5);
			}
		}
	}

	recalculateStatistics ();

	return Q_OK;
}

int QImage::changeLightnessAdd (const int argAspect) {
	if (argAspect == 0) {
		return Q_OK;
	}

	int i, j;
	__QBYTE shift = (__QBYTE)argAspect;
	__QBYTE maxRGB,	// maximum value of the RGB-component in order not to exceed 255 value
			minRGB;	// minimum value of the RGB-component in order not to exceed 255 value
	__QBYTE maxR, maxG, maxB,
			minR, minG, minB;

	minRGB = 0;
	maxRGB = 255;

	if (ignoreOutliers == true) {	// ignore too large or too small r,g,b components recalculation
		// just a cork
		maxR = _maxR;
		maxG = _maxG;
		maxB = _maxB;
		minR = _minR;
		minG = _minG;
		minB = _minB;
		// end of just a cork
	} else {
		maxR = _maxR;
		maxG = _maxG;
		maxB = _maxB;
		minR = _minR;
		minG = _minG;
		minB = _minB;
	}

	if (shift > 0) {
		if (shift + maxR > maxRGB) {
			shift = maxRGB - maxR;
		}
		if (shift + maxG > maxRGB) {
			shift = maxRGB - maxG;
		}
		if (shift + maxB > maxRGB) {
			shift = maxRGB - maxB;
		}
	} else {
		if (shift + minR < minRGB) {
			shift = minRGB - minR;
		}
		if (shift + minG < minRGB) {
			shift = minRGB - minG;
		}
		if (shift + minB < minRGB) {
			shift = minRGB - minB;
		}
	}	// if (shift > 0)

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			_image[i][j].r += (__QBYTE)shift;
			_image[i][j].g += (__QBYTE)shift;
			_image[i][j].b += (__QBYTE)shift;
		}
	}

	recalculateStatistics ();

	return Q_OK;
}

int QImage::histogramEqualization () {
	return qhisteq (_image, _width, _height);
}

int QImage::matlabImadjust () {
	if (isMonochrome ()) {
		double lowBorder, highBorder;	// saturation borders for monochrome image
		double intensityRange_1;
		__VECTOR_INT mapping;
		__VECTOR_DOUBLE hist;
		double tempSum;
		int i, j, temp;

		calculateHistogram (hist);
		// calculate borders so that 1% of pixels are left behind
		// low and high borders

		// calculate [lowBorder]
		tempSum = 0;
		for (i=0; i<256; i++) {
			tempSum += hist[i];
			if (tempSum > 0.005) {	// if more than 0.5%
				lowBorder = i;
				break;
			}
		}

		// calculate [highBorder]
		tempSum = 0;
		for (i=255; i>=0; i--) {
			tempSum += hist[i];
			if (tempSum > 0.005) {	// if more than 0.5%
				highBorder = i;
				break;
			}
		}

		// calculate borders so that 1% of pixels are left behind
		// low and high borders
//		lowBorder = _minG + 0.005 * (_maxG - _minG);
//		highBorder = _maxG - 0.005 * (_maxG - _minG);

		// calculate mapping array
		mapping.resize (256);
		intensityRange_1 = 1.0 / (double)(highBorder - lowBorder);
		for (i=0; i<256; i++) {
			if (i < lowBorder) {
				mapping [i] = 0;
				continue;
			}
			if (i > highBorder) {
				mapping[i] = 255;
				continue;
			}
			mapping[i] = (int)(255.0*(i-lowBorder) * intensityRange_1 + 0.5);
		}

		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				temp = mapping [ _image[i][j].g ];
				_image[i][j].r = _image[i][j].g = _image[i][j].b = (__QBYTE)temp;
			}
		}

	} else {	// color image case

		double lowR, lowG, lowB, highR, highG, highB;	// saturation borders for RGB image
		double intensityRangeR_1, intensityRangeG_1, intensityRangeB_1;
		__VECTOR_INT mappingR, mappingG, mappingB;
		__VECTOR_DOUBLE histR, histG, histB;
		const double bound = 0.005;
		double tempSum;
		int i, j;

		calculateHistogramR (histR);
		calculateHistogramG (histG);
		calculateHistogramB (histB);
		// calculate borders so that 1% of pixels are left behind
		// low and high borders

		// calculate [lowR]
		tempSum = 0;
		for (i=0; i<256; i++) {
			tempSum += histR[i];
			if (tempSum > bound) {	// if more than 0.5%
				lowR = i;
				break;
			}
		}
		// calculate [lowG]
		tempSum = 0;
		for (i=0; i<256; i++) {
			tempSum += histG[i];
			if (tempSum > bound) {	// if more than 0.5%
				lowG = i;
				break;
			}
		}
		// calculate [lowB]
		tempSum = 0;
		for (i=0; i<256; i++) {
			tempSum += histB[i];
			if (tempSum > bound) {	// if more than 0.5%
				lowB = i;
				break;
			}
		}

		// calculate [highR]
		tempSum = 0;
		for (i=255; i>=0; i--) {
			tempSum += histR[i];
			if (tempSum > bound) {	// if more than 0.5%
				highR = i;
				break;
			}
		}
		// calculate [highG]
		tempSum = 0;
		for (i=255; i>=0; i--) {
			tempSum += histG[i];
			if (tempSum > bound) {	// if more than 0.5%
				highG = i;
				break;
			}
		}
		// calculate [highB]
		tempSum = 0;
		for (i=255; i>=0; i--) {
			tempSum += histB[i];
			if (tempSum > bound) {	// if more than 0.5%
				highB = i;
				break;
			}
		}

		// calculate subsidiary variables
		intensityRangeR_1 = 1.0 / (highR - lowR);
		intensityRangeG_1 = 1.0 / (highG - lowG);
		intensityRangeB_1 = 1.0 / (highB - lowB);

		// calculate mapping arrays
		mappingR.resize (256);
		for (i=0; i<256; i++) {
			// mapping for R component
			if (i < lowR) {
				mappingR [i] = 0;
			} else if (i > highR) {
				mappingR [i] = 255;
			} else {
				mappingR [i] = (int)(255.0*(i-lowR) * intensityRangeR_1 + 0.5);
			}
		}	// for (i=0; i<256; i++)

		mappingG.resize (256);
		for (i=0; i<256; i++) {
			// mapping for G component
			if (i < lowG) {
				mappingG [i] = 0;
			} else if (i > highG) {
				mappingG [i] = 255;
			} else {
				mappingG [i] = (int)(255.0*(i-lowG) * intensityRangeG_1 + 0.5);
			}
		}	// for (i=0; i<256; i++)

		mappingB.resize (256);
		for (i=0; i<256; i++) {
			// mapping for B component
			if (i < lowB) {
				mappingB [i] = 0;
			} else if (i > highB) {
				mappingB [i] = 255;
			} else {
				mappingB [i] = (int)(255.0*(i-lowB) * intensityRangeB_1 + 0.5);
			}

		}	// for (i=0; i<256; i++)

		// adjust image
		for (i=0; i<_height; i++) {
			for (j=0; j<_width; j++) {
				_image[i][j].r = (__QBYTE)mappingR [ _image[i][j].r ];
				_image[i][j].g = (__QBYTE)mappingG [ _image[i][j].g ];
				_image[i][j].b = (__QBYTE)mappingB [ _image[i][j].b ];
			}
		}

	}	// if (isMonochrome ())

	recalculateStatistics ();

	return Q_OK;
}

int QImage::calculateNeighborhood (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD) const {
	clock_t start = clock(), finish;

	int i, j, i1, j1;
	int left, right, top, bottom;	// window borders
	int width, height, radius, square;
	double alpha1, alpha2;
	double temp;

	width = _width;
	height = _height;
	radius = (int)(argSize / 2);

	argM.resize (height);
	argD.resize (height);
	for (i=0; i<height; i++) {
		// define vertical window borders
		if (i > radius) {top = i-radius;} else {top = 0;}
		if (i < (height - radius)) {bottom = (i+radius);} else {bottom = (height-1);}

		argM[i].resize (width);
		argD[i].resize (width);
		for (j=0; j<width; j++) {
			// define horizontal window borders
			if (j > radius) {left = j-radius;} else {left = 0;}
			if (j < (width - radius)) {right = (j+radius);} else {right = (width-1);}

			square = (bottom - top + 1) * (right - left + 1);

			alpha1 = alpha2 = 0.0;
			for (i1=top; i1<=bottom; i1++) {
				for (j1=left; j1<=right; j1++) {
					temp = getPixel (i1, j1)->g;
					alpha1 += temp;
					alpha2 += temp*temp;
				}
			}
			alpha1 *= 0.003922 / (double)square;	// maximum 256 levels of brightness is assumed
			alpha2 *= 0.003922*0.003922 / (double)square;

			argM[i][j] = alpha1;
			argD[i][j] = alpha2 - alpha1*alpha1;
		}
	}

	finish = clock ();
	return (int)(finish - start);
}

int QImage::calculateNeighborhoodApprx (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD) const {
	clock_t start = clock(), finish;

	int i, j, i1, j1;
	int left, right, top, bottom;	// window borders
	int width, height, radius;
	double temp, sum1, sum2, sum1a, sum2a;
    double avL;

	width = _width;
	height = _height;
	radius = (int)(argSize / 2);

	avL = 0.003922 * _avBrightness;

	argM.resize (height);
	argD.resize (height);
	for (i=0; i<height; i++) {
		if (i > radius) {top = i-radius;} else {top = 0;}
		if (i < (height - radius)) {bottom = (i+radius);} else {bottom = (height-1);}

		argM[i].resize (width);
		argD[i].resize (width);
		for (j=0; j<width; j++) {
			if (j > radius) {left = j-radius;} else {left = 0;}
			if (j < (width - radius)) {right = (j+radius);} else {right = (width-1);}

			// 1-st step
			temp = 0.0;
			sum1 = sum2 = 0.0;
			for (i1=top; i1<=bottom; i1++) {
				temp = getPixel( i1, j )->g;
//				sum1 += w[ size-1 - bottom+i1 ] * temp;
				sum1 += temp;
				sum2 += temp * temp;
			}
			sum1 *= 0.003922 / (bottom - top + 1.0);
			sum2 *= 0.003922*0.003922 / (bottom - top + 1.0);

			// 2-nd step
			temp = 0.0;
			sum1a = sum2a = 0.0;
			for (j1=left; j1<=right; j1++) {
				temp = getPixel( i, j1 )->g;
//				sum1a += w[ size-1 - right+j1 ] * temp;
				sum1a += temp;
				sum2a += temp * temp;
			}
			sum1a *= 0.003922 / (right - left + 1.0);
			sum2a *= 0.003922 * 0.003922 / (right - left + 1.0);

			argM[i][j] = (0.33 * (sum1 + sum1a + avL));	// calculate mean L with respect to scaling
			argD[i][j] = 0.5 * (sum2 - sum1*sum1 + sum2a - sum1a*sum1a);
		}
	}

	finish = clock ();
	return (int)(finish - start);
}

int QImage::calculateNeighborhoodApprxOnlyLocal (const int argSize, __TABLE_DOUBLE &argM, __TABLE_DOUBLE &argD) const {
	clock_t start = clock(), finish;

	int i, j, i1, j1;
	int left, right, top, bottom;	// window borders
	int width, height, radius;
	double temp, sum1, sum2, sum1a, sum2a;

	width = _width;
	height = _height;
	radius = (int)(argSize / 2);

	argM.resize (height);
	argD.resize (height);
	for (i=0; i<height; i++) {
		if (i > radius) {top = i-radius;} else {top = 0;}
		if (i < (height - radius)) {bottom = (i+radius);} else {bottom = (height-1);}

		argM[i].resize (width);
		argD[i].resize (width);
		for (j=0; j<width; j++) {
			if (j > radius) {left = j-radius;} else {left = 0;}
			if (j < (width - radius)) {right = (j+radius);} else {right = (width-1);}

			// 1-st step
			temp = 0.0;
			sum1 = sum2 = 0.0;
			for (i1=top; i1<=bottom; i1++) {
				temp = _image[i1][j].g;
				sum1 += temp;
				sum2 += temp * temp;
			}
			sum1 *= 0.003922 / (bottom - top + 1.0);
			sum2 *= 0.003922*0.003922 / (bottom - top + 1.0);

			// 2-nd step
			temp = 0.0;
			sum1a = sum2a = 0.0;
			for (j1=left; j1<=right; j1++) {
				temp = _image[i][j1].g;
				sum1a += temp;
				sum2a += temp * temp;
			}
			sum1a *= 0.003922 / (right - left + 1.0);
			sum2a *= 0.003922 * 0.003922 / (right - left + 1.0);

			argM[i][j] = (0.5 * (sum1 + sum1a));	// calculate mean L with respect to scaling
			argD[i][j] = 0.5 * (sum2 - sum1*sum1 + sum2a - sum1a*sum1a);
		}
	}

	finish = clock ();
	return (int)(finish - start);
}

bool QImage::isMonochrome () {
	int i, j;

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			if (_image[i][j].b != _image[i][j].g || _image[i][j].b != _image[i][j].r) {
				return false;
			}
		}
	}

	return true;
}

int QImage::calculateHistogram (const char* argFilename) {
	__VECTOR_DOUBLE levels;
	QImage tempImage;
	QRGB black, *tempPixel;
	int i, j, square = _width * _height, barL;
	double maxLevel;

	levels.resize (256);
	levels.assign (256, 0);

	tempImage = (*this);
	if (!this->isMonochrome ()) {
		tempImage.toGrayscale ();
	}

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			levels[ tempImage.getPixel(i, j)->g ]++;
		}
	}

	for (i=0; i<256; i++) {
		levels[i] = levels[i] / (double)square;
	}
	maxLevel = (*(max_element( levels.begin(), levels.end() )));

	tempImage.setDimensions (536, 474);
	tempImage.fillWhite ();

	black.r = black.g = black.b = 0;
	for (j=0; j<256; j++) {
		barL = (int)((levels[j]) * 400.0 / maxLevel + 0.5);
		for (i=400-barL; i<400; i++) {
			(*tempImage.getPixel(12+i, 12+2*j)) = black;
			(*tempImage.getPixel(12+i, 12+2*j+1)) = black;
		}

		// fill brightness distribution bar
		for (i=415; i<465; i++) {
			tempPixel = tempImage.getPixel(i, 12+2*j);
			tempPixel->r = tempPixel->g = tempPixel->b = (__QBYTE)j;

			tempPixel = tempImage.getPixel(i, 12+2*j+1);
			tempPixel->r = tempPixel->g = tempPixel->b = (__QBYTE)j;
		}
	}

	// draw a separating line between histogram and brightness diagram
	for (j=0; j<536; j++) {
		tempPixel = tempImage.getPixel(413, j);
		tempPixel->r = tempPixel->g = tempPixel->b = 64;

		//tempPixel = tempImage.getPixel(414, j);
		//tempPixel->r = tempPixel->g = tempPixel->b = 64;
	}

	// draw horizontal borders
	for (j=10; j<526; j++) {
		tempPixel = tempImage.getPixel(10, j);
		tempPixel->r = tempPixel->g = tempPixel->b = 170;

		tempPixel = tempImage.getPixel(466, j);
		tempPixel->r = tempPixel->g = tempPixel->b = 170;
	}
	// draw vertical borders
	for (i=10; i<466; i++) {
		tempPixel = tempImage.getPixel(i, 10);
		tempPixel->r = tempPixel->g = tempPixel->b = 170;

		tempPixel = tempImage.getPixel(i, 525);
		tempPixel->r = tempPixel->g = tempPixel->b = 170;
	}

	tempImage.writeToFile (argFilename);

	return Q_OK;
}

double QImage::calculateEntropy () {
	double entropy=0, temp;
	int i, j, size = _height*_width;
	__VECTOR_INT hist;

	hist.resize (256);
	hist. assign (256, 0);
	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			hist [ _image[i][j].g ]++;
		}
	}

	_maxBrightness = 0;
	for (i=255; i>=0; i--) {
		temp = (double)hist[i] / size;
		entropy -= temp* log(temp);

		if (_maxBrightness == 0 && hist[i] > 0) {
			_maxBrightness = i;
		}
	}

	entropy = 0.0;
	for (i=0; i<256; i++) {
		temp = (double)hist[i] / size;
		entropy -= temp* log(temp);
	}

	return entropy;
}


int QImage::calculateHistogram (__VECTOR_DOUBLE& argRes) {
	QImage tempImage;
	int i, j;
	double square_1 = 1.0 / (_width * _height);
//	double maxLevel;

	argRes.assign (256, 0);

	tempImage = (*this);
	if (!this->isMonochrome ()) {
		tempImage.toGrayscale ();
	}

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argRes[ tempImage.getPixel(i, j)->g ]++;
		}
	}

	
	for (i=0; i<256; i++) {
		argRes[i] = argRes[i] * square_1;
	}
//	maxLevel = (*(max_element( argRes.begin(), argRes.end() )));

	return Q_OK;
}

int QImage::calculateHistogramR (__VECTOR_DOUBLE& argRes) {
	int i, j;
	double square_1 = 1.0 / (_width * _height);
//	double maxLevel;

	argRes.assign (256, 0);

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argRes[ _image[i][j].r ]++;
		}
	}

	for (i=0; i<256; i++) {
		argRes[i] = argRes[i] * square_1;
	}
//	maxLevel = (*(max_element( argRes.begin(), argRes.end() )));

	return Q_OK;
}

int QImage::calculateHistogramG (__VECTOR_DOUBLE& argRes) {
	int i, j;
	double square_1 = 1.0 / (_width * _height);
//	double maxLevel;

	argRes.assign (256, 0);

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argRes[ _image[i][j].g ]++;
		}
	}

	for (i=0; i<256; i++) {
		argRes[i] = argRes[i] * square_1;
	}
//	maxLevel = (*(max_element( argRes.begin(), argRes.end() )));

	return Q_OK;
}

int QImage::calculateHistogramB (__VECTOR_DOUBLE& argRes) {
	int i, j;
	double square_1 = 1.0 / (_width * _height);
//	double maxLevel;

	argRes.assign (256, 0);

	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argRes[ _image[i][j].b ]++;
		}
	}

	for (i=0; i<256; i++) {
		argRes[i] = argRes[i] * square_1;
	}
//	maxLevel = (*(max_element( argRes.begin(), argRes.end() )));

	return Q_OK;
}

int QImage::getPlaneR (QImage* argImg) {
	argImg->setDimensions (_width, _height);

	int i, j;
	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argImg->_image[i][j].r = argImg->_image[i][j].g = argImg->_image[i][j].b = _image[i][j].r;
		}
	}

	return Q_OK;
}

int QImage::getPlaneG (QImage* argImg) {
	argImg->setDimensions (_width, _height);

	int i, j;
	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argImg->_image[i][j].r = argImg->_image[i][j].g = argImg->_image[i][j].b = _image[i][j].g;
		}
	}

	return Q_OK;
}

int QImage::getPlaneB (QImage* argImg) {
	argImg->setDimensions (_width, _height);

	int i, j;
	for (i=0; i<_height; i++) {
		for (j=0; j<_width; j++) {
			argImg->_image[i][j].r = argImg->_image[i][j].g = argImg->_image[i][j].b = _image[i][j].b;
		}
	}

	return Q_OK;
}


void QImage::setDimensions (const int argW, const int argH) {
	int i, j;
	QRGB black;
	black.r = black.g = black.b = 0;

	eraseImage ();
	_image = new QRGB* [argH];
	_height = argH;
	_width = argW;
	for (i=0; i<argH; i++) {
		_image[i] = new QRGB [argW];
		for (j=0; j<argW; j++) {
			_image[i][j] = black;
		}
	}
}


////////////////////////////////////////////////////////////////////////////
// QDoubleArray class definition
//
QDoubleArray::QDoubleArray () : Items(NULL), Size(0) {
}

QDoubleArray::QDoubleArray (const int size) {
	Size = size;
	Items = new double [Size];
}

QDoubleArray::QDoubleArray (const int size, const double* items) {
	Size = size;
	Items = new double [Size];

	for (int i=Size-1; i>=0; i--) {
		Items[i] = items[i];
	}
}

QDoubleArray::QDoubleArray (const QDoubleArray& argArray) {
	Size = argArray.Size;
	Items = new double [Size];

	for (int i=Size-1; i>=0; i--) {
		Items[i] = argArray.Items[i];
	}
}

QDoubleArray::~QDoubleArray () {
	if (Items != NULL) {
		delete[] Items;
	}
	Items = NULL;
	Size = 0;
}

QDoubleArray& QDoubleArray::operator= (const QDoubleArray& argArray) {
	Size = argArray.Size;
	Items = new double [Size];

	for (int i=Size-1; i>=0; i--) {
		Items[i] = argArray.Items[i];
	}
	return *this;
}

double& QDoubleArray::operator [] (const int index) const {
	return Items[index];
}

int QDoubleArray::SetSize (const int size) {
	if (Items != NULL) {
		delete[] Items;
	}

	Size = size;
	Items = new double [Size];
	return 0;
}

int QDoubleArray::Reset () {
	if (Size != 0) {
		delete[] Items;
	}
	Size =  0;
	Items = NULL;
	return 0;
}

////////////////////////////////////////////////////////////////////////////
// QDoubleArray class definition
QDoubleArrayNode::QDoubleArrayNode () : Index(0), Next(NULL) {
}

QDoubleArrayNode::QDoubleArrayNode (const int size, const double* items, const int index) : Next(NULL) {
	Index = index;
	Size = size;
	Items = new double [Size];

	for (int i=Size-1; i>=0; i--) {
		Items[i] = items[i];
	}
}

QDoubleArrayNode::~QDoubleArrayNode () {
	Index = 0;
	Next = NULL;
}

int QDoubleArrayNode::PrintInfo () {
	cout<<"Node "<<Index<<" : ";
	for (int i=0; i<Size; i++) {
		cout<<Items[i]<<" ";
	}
	cout<<endl;
	return 0;
}

////////////////////////////////////////////////////////////////////////////
// QDoubleArrayList class definition
QDoubleArrayList::QDoubleArrayList () : First(NULL), Last(NULL), Count(0) {
}

QDoubleArrayList::~QDoubleArrayList () {
//	cout<<"~QDoubleArrayList...";
//	cout.flush ();
	if (First != NULL) {
		QDoubleArrayNode *tempNode = First->Next;
		while (tempNode != NULL) {
			delete First;
			First = tempNode;
			tempNode = First->Next;
		}
		delete First;
	}
	First = Last = NULL;
	Count = 0;
//	cout<<"complete\n";
//	cout.flush ();
}

int QDoubleArrayList::Add (const int size) {
	if (First != NULL) {
		Last->Next = new QDoubleArrayNode ();
		Last = Last->Next;
	} else {	// if empty list
		First = Last = new QDoubleArrayNode ();
	}
	Last->SetSize (size);
	Last->Index = Count;
	Last->Next = NULL;
	Count++;
	return 0;
}

int QDoubleArrayList::Add (const int size, const double *items) {
	if (First != NULL) {
		Last->Next = new QDoubleArrayNode (size, items, Count);
		Last = Last->Next;
	} else {	// if empty list
		First = Last = new QDoubleArrayNode (size, items, Count);
	}
	Last->Index = Count;
	Last->Next = NULL;
	Count++;
	return 0;
}

int QDoubleArrayList::Add (const QDoubleArrayNode& argArray) {
	if (First != NULL) {
		Last->Next = new QDoubleArrayNode (argArray);
		Last = Last->Next;
	} else {	// if empty list
		First = Last = new QDoubleArrayNode (argArray);
	}
	Last->Index = Count;
	Last->Next = NULL;
	Count++;
	return 0;
}

QDoubleArray& QDoubleArrayList::operator [] (const int index) {
	QDoubleArrayNode *tempArray = First;
	while (tempArray != NULL) {
		if (tempArray->Index == index) {
			return *tempArray;
		}
		tempArray = tempArray->Next;
	}
	return *tempArray;
}

int QDoubleArrayList::Reset () {
	if (First != NULL) {
		QDoubleArrayNode *tempNode = First->Next;
		while (tempNode != NULL) {
			delete First;
			First = tempNode;
			tempNode = First->Next;
		}
		delete First;
	}
	First = Last = NULL;
	Count = 0;

	return 0;
}

int QDoubleArrayList::PrintInfo () {
	QDoubleArrayNode *tempArray = First;

	while (tempArray != NULL) {
		tempArray->PrintInfo ();
		tempArray = tempArray->Next;
	}
	return 0;
}
////////////////////////////////////////////////////////////////////////////
// Classes for work with config files
// General config-file structure consideration:
//		Parameter - Includes name and value.
//								Name should begin from symbol '$' and includes chars
//								[a..z, A..Z, 0..9, _ ]. There should be bo spaces between
//								'$' and parameter name. Name is ended when space or equation
//								symbol encountered. Parameter value should be in quotes.
//								Example : $Name = "Value"
//													$Name="Value"
//		Comments - comments begin from symbol '#' and goes until the end of line.
//							 Example: #Comments
//							 Symbol '#' inside parameter value doesn't mark comments beginning
//							 Example: $Name = "Value # comments" - defines parameter 'Name'
//												and its value 'Value # comments'
// One and the same string can include both comments and parameter value
//		if parameter precedes comments.
//		Example: $Name="Value"		# Definition of parameter 'Name'
// If parameter value includes quotes then single quotes symbol should be used
//		instead of double quotes symbol to avoid possible confusions
//		Example: $Name="Value is '1' for all" - correct
//						 $Name="Value is "1" for all"	- error

// Class QConfigParameter functions
QConfigParameter::QConfigParameter () :
Name(NULL), cValue(NULL), nValue(0), Type (CPT_TEXT) {
/*	cout<<"QConfigParameter::QConfigParameter\n";
	cout.flush ();/**/
}

QConfigParameter::QConfigParameter (const char* name) :  cValue(NULL), nValue(0), Type (CPT_TEXT) {
	qstrcpy (Name, name);	// copy defined parameter name into class variable
}

QConfigParameter::QConfigParameter (const char* name, const QConfigParameterType cpt, const char* value) {
	qstrcpy (Name, name);
	Type = cpt;
	qstrcpy (cValue, value);
}

QConfigParameter::QConfigParameter (const QConfigParameter& argParam) {
	qstrcpy (Name, argParam.Name);
	qstrcpy (cValue, argParam.cValue);
	nValue = argParam.nValue;
	Type = argParam.Type;
}

QConfigParameter::~QConfigParameter () {
/*	cout<<"Deleting parameter "<<Name<<"\n";
	cout.flush ();/**/
	if (Name != NULL) delete[] Name;
	if (cValue != NULL) delete[] cValue;
	nValue = 0;
	Type = CPT_UNKNOWN;
}

QConfigParameter& QConfigParameter::operator= (const QConfigParameter& argParam) {
	qstrcpy (Name, argParam.Name);
	qstrcpy (cValue, argParam.cValue);
	nValue = argParam.nValue;
	Type = argParam.Type;

	return *this;
}

QConfigParameter& QConfigParameter::operator= (const char* argValue) {
	qstrcpy (cValue, argValue);
	if (Type == CPT_NUMBER) {
		char2dbl (nValue, cValue);
	}
	return *this;
}

QConfigParameter& QConfigParameter::operator= (const double argValue) {
	nValue = argValue;
	dbl2char (cValue, nValue);
	return *this;
}

ostream& operator<< (ostream& stream, const QConfigParameter& argParam) {
	if (argParam.Type == CPT_TEXT) {
		stream<<"$"<<argParam.Name<<" = \""<<argParam.cValue<<"\"";
	} else {	// Type == CPT_NUMBER
		stream<<"$"<<argParam.Name<<" = \""<<argParam.nValue<<"\"";
	}
	return stream;
}

int QConfigParameter::SetName (const char *name) {
	qstrcpy (Name, name);
	return 0;
}

char* QConfigParameter::GetValueC () {
//	char* str;
//	qstrcpy (str, cValue);
	return cValue;
}

double QConfigParameter::GetValueN () {
	if (cValue != NULL) {
		char2dbl (nValue, cValue);
//		cout<<nValue<<endl;
//		cout<<cValue<<endl;
//		nValue = atof (cValue);
	}
//	qstrcpy (str, cValue);
	return nValue;
}

int QConfigParameter::SetValue (const char *value) {
	qstrcpy (cValue, value);
	return 0;
}

int QConfigParameter::SetValue (const double value) {
	nValue = value;
	return 0;
}

int QConfigParameter::SetType (const QConfigParameterType argType) {
	Type = argType;
	if (Type == CPT_NUMBER && cValue!=NULL) {
		char2dbl (nValue, cValue);
	} else {
		nValue = 0.0;
		qstrcpy (cValue, "0.0");
	}
	return 0;
}

// Class QConfigIO functions
QConfigIO::QConfigIO () {
}

QConfigIO::~QConfigIO () {
}

int QConfigIO::ReadFromStream (const istream& fin) {
	return 0;
}

int QConfigIO::ReadFromFile (const char* filename) {
	char a, name[80], value[256];
	QConfigParameter tempParam;
	ifstream fin(filename);

	if (fin.is_open()) {
		a = (char)fin.get ();
		while (a != EOF) {
			if (a == '$') {	// read name and value
				ReadParameter (fin, name, value);
				tempParam.SetName (name);
				tempParam.SetValue (value);
				tempParam.SetType (CPT_TEXT);
				Params.push_back (tempParam);
//				Params.Add (name, CPT_TEXT, value);
/*				cout<<"read name = "<<name<<endl
						<<"read value = "<<value<<endl;/**/
			} else if (a == '#') {
				NextLine (fin);
			}
			a = (char)fin.get ();
		}	// end of while (a != EOF)
	} else {
		cout<<"Error (QConfigIO::ReadFromFile):\n\tCan't open specified config-file\n";
		exit (0);
	}
	fin.close ();
	return 0;
}

int QConfigIO::ReadParameter (ifstream& fin, char* name, char* value) {
	char a=(char)fin.get();
	int i=0;

	// read name
	do {
		name[i] = a;
		a = (char)fin.get ();
		i++;
	} while (a!=0x20 && a!='=' && a!='\t');
	name[i] = 0;	// write null-terminator

	// go until parameter value
	while (a != '\"') {
		a = (char)fin.get ();
	}
	a = (char)fin.get ();
	i=0;
	while (a != '\"') {
		value[i] = a;
		a = (char)fin.get ();
		i++;
	}
	value[i] = 0;	// write null-terminator

	return 0;
}

int QConfigIO::NextLine (ifstream& fin) {
	char a=0;

	while (a!='\n' && a!=EOF) {
		a = (char)fin.get ();
	}

	return 0;
}
int QConfigIO::ReadName (ifstream& fin, char*& name) {
	return 0;
}

int QConfigIO::ReadValue (ifstream& fin, char*& value) {
	return 0;
}/**/

int QConfigIO::WriteToFile (const char* filename) {
	ofstream fout (filename);
	__QCONFIG_PARAMETER_LIST::iterator iter = Params.begin ();
//	QConfigParameterNode *tempParam = Params.GetFirst();

	while (iter != Params.end ()) {
		fout<<(*iter)<<endl;
		iter++;
	}

	fout.close ();
	return 0;
}

int QConfigIO::WriteToStream (const ostream& fout) {
	return 0;
}

// Both matrices are supposed to be square and to have one and the same size
// There is no check for "equal sizes" condition, so beware and be careful!
//
const __TABLE_BYTE operator* (const __TABLE_BYTE& argL, const __TABLE_BYTE& argR) {
	__TABLE_BYTE res;
	if (!argL.empty() && !argR.empty()) {
		int size = (int)argL.size();
		int i, j, k, temp;

		// resize resulting matrix
		res.resize (size);
		for (i=0; i<size; ++i) {
			res[i].resize (size);
		}

		// multiply matrices and write the result into
		// [res] matrix
		for (i=0; i<size; ++i) {
			for (j=0; j<size; ++j) {
				temp = 0;
				for (k=0; k<size; ++k) {
					temp += argL[i][k] * argR[k][j];
				}
				res[i][j] = temp;
			}
		}
	}
	return res;
}
